'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vue = require('vue');
var core = require('@weamd/core');
var exception = require('@weamd/exception');
var prosemirrorModel = require('prosemirror-model');
var nanoid$1 = require('nanoid');

/* Copyright 2021, Milkdown by Mirone. */
const rendererKey = Symbol();
const useGetEditor = (getEditor) => {
    const renderVue = vue.inject(rendererKey, () => {
        throw exception.vueRendererCallOutOfScope();
    });
    const { dom, loading, editor: editorRef } = vue.inject(editorInfoCtxKey, {});
    const lock = vue.ref(false);
    vue.onMounted(() => {
        if (!dom.value)
            return;
        const editor = getEditor(dom.value, renderVue);
        if (!editor)
            return;
        if (lock.value)
            return;
        loading.value = true;
        lock.value = true;
        editor
            .create()
            .then((editor) => {
            editorRef.value = editor;
            return;
        })
            .finally(() => {
            loading.value = false;
            lock.value = false;
        })
            .catch((e) => console.error(e));
    });
    vue.onUnmounted(() => {
        var _a, _b, _c;
        const view = (_a = editorRef.value) === null || _a === void 0 ? void 0 : _a.action((ctx) => ctx.get(core.editorViewCtx));
        const root = (_b = editorRef.value) === null || _b === void 0 ? void 0 : _b.action((ctx) => ctx.get(core.rootCtx));
        (_c = root === null || root === void 0 ? void 0 : root.firstChild) === null || _c === void 0 ? void 0 : _c.remove();
        view === null || view === void 0 ? void 0 : view.destroy();
    });
};

const EditorComponent = vue.defineComponent({
    name: 'milkdown-dom-root',
    setup: (props, { slots }) => {
        useGetEditor(props.editor);
        const ctx = vue.inject(editorInfoCtxKey, {});
        if (props.editorRef) {
            props.editorRef.get = () => ctx.editor.value;
            props.editorRef.dom = () => ctx.dom.value;
        }
        return () => { var _a; return vue.h("div", { ref: ctx.dom }, (_a = slots['default']) === null || _a === void 0 ? void 0 : _a.call(slots)); };
    },
});
EditorComponent['props'] = ['editor', 'editorRef'];

const nodeMetadata = Symbol();
const useNodeCtx = () => vue.inject(nodeMetadata);
const VueNodeContainer = vue.defineComponent({
    name: 'milkdown-node-container',
    setup: ({ node, view, getPos, decorations, ctx, as }, context) => {
        vue.provide(nodeMetadata, {
            ctx,
            node,
            view,
            getPos,
            decorations,
        });
        return () => { var _a, _b; return vue.h(as, { 'data-view-container': true }, (_b = (_a = context.slots)['default']) === null || _b === void 0 ? void 0 : _b.call(_a)); };
    },
});
VueNodeContainer['props'] = ['ctx', 'editor', 'node', 'view', 'getPos', 'decorations', 'as'];
const Content = vue.defineComponent({
    name: 'milkdown-content',
    setup: ({ isInline }) => {
        return () => (isInline ? vue.h("span", { "data-view-content": true }) : vue.h("div", { "data-view-content": true }));
    },
});
Content['props'] = ['isInline'];

const nanoid = nanoid$1.customAlphabet('abcedfghicklmn', 10);
const createVueView = (addPortal, removePortalByKey) => (component, options = {}) => (ctx) => (node, view, getPos, decorations) => new VueNodeView(ctx, component, addPortal, removePortalByKey, options, node, view, getPos, decorations);
class VueNodeView {
    constructor(ctx, component, addPortal, removePortalByKey, options, node, view, getPos, decorations) {
        var _a, _b;
        this.ctx = ctx;
        this.component = component;
        this.addPortal = addPortal;
        this.removePortalByKey = removePortalByKey;
        this.options = options;
        this.view = view;
        this.getPos = getPos;
        this.getPortal = () => {
            const CustomComponent = this.component;
            const elementName = this.options.as ? this.options.as : this.isInlineOrMark ? 'span' : 'div';
            return vue.markRaw(vue.defineComponent({
                name: 'milkdown-portal',
                setup: () => {
                    return () => (vue.h(vue.Teleport, { key: this.key, to: this.teleportDOM },
                        vue.h(VueNodeContainer, { as: elementName, ctx: this.ctx, node: this.node, view: this.view, getPos: this.getPos, decorations: this.decorations },
                            vue.h(CustomComponent, null,
                                vue.h(Content, { isInline: this.isInlineOrMark })))));
                },
            }));
        };
        this.selectNode = (_a = this.options) === null || _a === void 0 ? void 0 : _a.selectNode;
        this.deselectNode = (_b = this.options) === null || _b === void 0 ? void 0 : _b.deselectNode;
        this.key = nanoid();
        this.node = vue.ref(node);
        this.decorations = vue.ref(decorations);
        const elementName = options.as ? options.as : this.isInlineOrMark ? 'span' : 'div';
        this.teleportDOM = document.createElement(elementName);
        this.renderPortal();
    }
    get isInlineOrMark() {
        return this.node.value instanceof prosemirrorModel.Mark || this.node.value.isInline;
    }
    get dom() {
        return (this.teleportDOM.firstElementChild || this.teleportDOM);
    }
    get contentDOM() {
        if (this.node instanceof prosemirrorModel.Node && this.node.isLeaf) {
            return undefined;
        }
        return this.teleportDOM.querySelector('[data-view-content]') || this.dom;
    }
    renderPortal() {
        if (!this.teleportDOM)
            return;
        const Portal = this.getPortal();
        this.addPortal(Portal, this.key);
        const instance = getRootInstance();
        if (instance) {
            instance.update();
        }
    }
    destroy() {
        var _a, _b;
        (_b = (_a = this.options).destroy) === null || _b === void 0 ? void 0 : _b.call(_a);
        this.teleportDOM.remove();
        this.removePortalByKey(this.key);
    }
    ignoreMutation(mutation) {
        if (this.options.ignoreMutation) {
            return this.options.ignoreMutation(mutation);
        }
        if (!this.dom || !this.contentDOM) {
            return true;
        }
        if (this.node instanceof prosemirrorModel.Node) {
            if (this.node.isLeaf || this.node.isAtom) {
                return true;
            }
        }
        if (mutation.type === 'selection') {
            return false;
        }
        if (this.contentDOM === this.dom) {
            return false;
        }
        if (this.contentDOM.contains(mutation.target)) {
            return false;
        }
        return true;
    }
    update(node, decorations, innerDecorations) {
        const innerUpdate = () => {
            var _a, _b;
            if (this.options.update) {
                const result = (_b = (_a = this.options).update) === null || _b === void 0 ? void 0 : _b.call(_a, node, decorations, innerDecorations);
                if (result != null) {
                    return result;
                }
            }
            if (this.node.value.type !== node.type) {
                return false;
            }
            if (node === this.node.value && this.decorations.value === decorations) {
                return true;
            }
            this.node.value = node;
            this.decorations.value = decorations;
            return true;
        };
        const shouldUpdate = innerUpdate();
        return shouldUpdate;
    }
}

/* Copyright 2021, Milkdown by Mirone. */
const rootInstance = {
    instance: null,
};
const getRootInstance = () => {
    return rootInstance.instance;
};
const editorInfoCtxKey = Symbol();
const refDeprecatedInfo = `
@weamd/vue:
Passing ref to VueEditor will soon be deprecated, please use:

const { editor, getInstance, getDom, loading } = useEditor(/* creator */);

effect(() => {
    if (!loading) {
        const editor = getInstance();
        const rootDOM = getDom();
    }
})

<VueEditor editor={editor} />
`;
const compositionDeprecatedInfo = `
@weamd/vue:
Passing editor directly to VueEditor will soon be deprecated, please use:

const { editor } = useEditor(/* creator */);

<VueEditor editor={editor} />
`;
const VueEditor = vue.defineComponent({
    name: 'milkdown-vue-root',
    setup: (props) => {
        const portals = vue.shallowReactive([]);
        const instance = vue.getCurrentInstance();
        vue.onBeforeMount(() => {
            rootInstance.instance = instance
                .ctx._;
        });
        vue.onUnmounted(() => {
            rootInstance.instance = null;
        });
        const addPortal = vue.markRaw((component, key) => {
            portals.push([key, component]);
        });
        const removePortalByKey = vue.markRaw((key) => {
            const index = portals.findIndex((p) => p[0] === key);
            portals.splice(index, 1);
        });
        const renderVue = createVueView(addPortal, removePortalByKey);
        vue.provide(rendererKey, renderVue);
        const usingDeprecatedCompositionAPI = Object.hasOwnProperty.call(props.editor, 'getInstance');
        const { getEditorCallback, dom, editor, loading } = usingDeprecatedCompositionAPI
            ? // @ts-expect-error deprecated old composition API
                props.editor.editor
            : props.editor;
        vue.effect(() => {
            if (usingDeprecatedCompositionAPI) {
                console.warn(compositionDeprecatedInfo);
            }
            if (props.editorRef) {
                console.warn(refDeprecatedInfo);
            }
        });
        vue.provide(editorInfoCtxKey, {
            dom,
            editor,
            loading,
        });
        return () => {
            const portalElements = portals.map(([id, P]) => vue.h(P, { key: id }));
            return (vue.h(EditorComponent, { editorRef: props.editorRef, editor: getEditorCallback.value }, portalElements));
        };
    },
});
VueEditor['props'] = ['editor', 'editorRef'];

/* Copyright 2021, Milkdown by Mirone. */
const useEditor = (getEditor) => {
    const dom = vue.ref(null);
    const editor = vue.ref();
    const loading = vue.ref(true);
    const getEditorCallback = vue.ref((...args) => getEditor(...args));
    return {
        loading,
        getInstance: () => editor.value,
        getDom: () => dom.value,
        editor: {
            getEditorCallback,
            dom,
            editor,
            loading,
        },
    };
};

exports.Content = Content;
exports.VueEditor = VueEditor;
exports.VueNodeContainer = VueNodeContainer;
exports.editorInfoCtxKey = editorInfoCtxKey;
exports.getRootInstance = getRootInstance;
exports.nodeMetadata = nodeMetadata;
exports.useEditor = useEditor;
exports.useNodeCtx = useNodeCtx;
//# sourceMappingURL=index.cjs.map
